# Chrome Dev Editor Release Notes

## M19 (0.19.x, December 30, 2014)

### Improved features
- CSP fixer

    - `*.html.pre_csp` files are no longer generated from the original .html's
    - add CoffeeScript to the script types supported by the CSP fixer

- Project templates

    - pined down Polymer version in the templates to `latest` (was `master`)
    - Normalization in Chrome app templates manifests
    - fixed typos and made more uniform the Chrome App manifests generated by the project template types that have them

- Chrome App manifest validator: added `alwaysOnTopWindows` to the list of recognized permissions

### Bug fixes
- fix: Bower would fail to run on a bower_components directory and printed an exception to the console
- fix: Few compile-time warnings
- fix  File filter and search view are broken

## M18 (0.18.x, December 1, 2014)

### Improved features
- Bower:

    - support package specifications with just a version or version range and path omitted, e.g. `"package-x": "1.2.3"`, `"package-y": "~1.2.3"`: the path is resolved using Bower registry based on the package name
    - support `"*"` as a package specification, e.g. `"package-x": "*"`: the path is resolved as above; the tag resolves to the latest stable
    - support packages whose GitHub repositories have moved to a different GitHub organization (so their URL redirects from `https://github.com/OldOrg/package` to `https://github.com/NewOrg/package`)

### Bug fixes
- Bower: too frequent or large `bower install/update` jobs would exceed the GitHub API access rate limit, locking the user out of Bower for a 1-hour timeout period
- when deleting a folder from disk on Windows, CDE might delete the contents of a subfolder represented by a junction point, rather then just delete the junction itself
- the editor was very sluggish on very large files with long lines (e.g. minified package sources)
- editor focus would be lost when switching between files via Ctrl-Tab
- sporadic installation failures on Bit9-protected Windows machines should be slightly less probable now, although there is no complete solution yet

### Other changes
- pinned the version of Polymer packages in the Polymer-related templates to `latest` (was `master`): this should result in more stability
- added more USB devices to the supported list (LG Optimus v3)
- some UI adjustments and refinements

## M17 (0.17.3412, November 4, 2014)

### New features
- improved Bower support

    - added support for most of the [semantic version range formats](http://semver.org/) under `"dependencies"` in `bower.json`:

        - operators `<`, `<=`, `>`, `>=`, `~` ("approximate"), `^` ("latest compatible")
        - special tag `latest` ("latest stable tag")
        - unspecified version defaults to `latest`)
        - examples: `"Polymer/polymer#>=0.4.1 <0.4.3"`, `"Polymer/polymer#^0.4.1"`, `"Polymer/polymer#~0.3.0"`, `"Polymer/polymer#latest"`

    - added support for `"*"` package paths in `bower.json`: such paths will be resolved using Bower registry based on the package name; e.g. `"codemirror": "*"`
    - the destination directory for downloaded packages can be configured via `"directory"` field in hierarchically traversed `.bowerrc` files (other fields are currently ignored); see [Bower configuration page](http://bower.io/config/)

- alpha version of [Polymer Designer](https://www.polymer-project.org/tools/designer/) integration into CDE; some notes:

    - enabled for all HTML files, but meaningful only for HTMLs with a `<polymer-element>` at the top
    - very sensitive to the input: may fail to render a design if the internal parsing fails; improvements on the way
    - therefore, the primary indended use for now is to jump-start a new Polymer element from scratch, get it to a workable state, then continue more advanced development manually
    - accessible via a new context menu item in the file tree and via a new floating action button (FAB) in the editor
    - to start a new element design, simply create a new empty HTML and click the Polymer FAB in it
    - generates only the `.html` source for a design; you will need to manually add any dependencies the design requires to a sibling `bower.json` and run `Bower Install` on it
    - the undo stack is properly updated with the generated output: the original source can be restored via the usual `Undo` action
    - known limitations:

        - `<link>` tags importing external sources are ignored, including outline stylesheets and polymer components not natively known to the Designer (i.e. ones not found it its design palette); use inline tags wherever possible
        - lists of CSS selectors before a single rule are not supported: use duplicate rules for each individual selector
        - HTML tags containing both text content and sub-tags are parsed incorrectly: the text contents is lost; e.g. `Some text` in `<p>Some text <a>some link</a></p>` will be lost on import

### Project templates
- the `New Project` dialog can now also be accessed via a new floating action button (FAB) at the bottom of the file tree view
- the much improved `Dart Web App` and the newly added `Dart Package` project templates are now based on [Stagehand](http://stagehand.pub/) - a new set of best-of-breed prescriptive templates for creating Dart projects
- Polymer-related project templates now depend on the latest stable versions of Polymer components (depended on `master` before)
- fixed/upgraded some previously/recently broken templates

### Other changes
- significant improvements in the editing speed and responsiveness!
- add options to the `Search/Replace` dialog (regular expressions, case-sensitive, whole words only)
- enabled syntax highighting for `.sql` and `.sqlite` file extensions

- UI tweaks:

    - improved the `Live Deploy` dialog's UI
    - restricted resizing of the commit message text area in the `Git Commit` dialog to vertical only (could be resized horizontally before)
    - added 11 light color themes for the editor

### Bug fixes
- the undo stack in the editor would reset when the rereading the contents of an externally modified file, making it impossible to go back to the original code
- source navigation would become broken when an involved project got deleted

## M16 (0.16.3226, October 1, 2014)
### New features
- added the 'Live Deploy' mode when pushing to mobile: see your changes reflected on the device in real time!
- added real-time validation for Chrome apps' manifests
- enabled running Bower commands on a project's subdirectories that contain their own bower.json -- not only the top level one
- enabled 'Refactor for CSP' context menu for any HTML files or folder(s) with HTMLs -- not only package directories

### Other changes
- adjusted the goto line component to be more in-line with material design
- faster handling of large files (> 500 K); in particular, saving is faster
- enabled syntax highlighting for *.html.pre_csp files (backups of original *.html's created by 'Refactor for CSP')
- made possible editing of files in packages via package-files-are-editable developer flag
- UI tweaks:

    - upgraded to Roboto 2 font
    - moved the 'Toggle Outline' button to the toolbar
    - moved the status/progress indicator to the bottom toolbar instead of the blue bubble
    - display a warning dialog when a file is dragged onto a folder where a file with the same name already exists
    - hide "Properties..." context menu item when multiple resources are selected
    - added some missing tooltips to UI elements

### Bug fixes
- fixed: warning markers obscured error markers in the same line in the editor

## M15 (0.15.x, September 2, 2014)
### New features:
- more dark themes (and the ability for the user to change the editor theme)
- new project template: "JavaScript Chrome App (using Polymer paper elements)"
- new context menu item on project/folder/file(s) to fix certain CSP violations (enables the above)
- as-you-type validation of manifest.json files (for Chrome apps)

### Other changes:
- fixed an issue with cloning an empty Git repository
- fixed several issues with 'Pub Get' and 'Pub Update'
- USB deployment speed improvements
- fixed a Bower issue where just the top-level files of a package would be downloaded (subdirectories were skipped)

## M14 (0.14.2, August 5, 2014)
### New features:
- the Polymer template now uses [Paper elements](http://www.polymer-project.org/docs/elements/material.html)
- added a new Dart Chrome App project type
- added a search in files feature
- improved the performance of the files view
- improved the performance of our Bower support

### And other changes:
- the main folder for projects can now be changed
- the outline view is now resizable
- the status message become translucent when the mouse hovers over it
- bug fixes to the Dart support, for both Dart analysis and compiling to JavaScript
- improved some error messages for pub and webstore publish
- fixed some history navigation issues
